function CompareIntegrationMethods(im, p, n_lines)
    % Function that plots a comparison between the different values of 
    % param.computeline.method, by plotting n_lines random lines from the
    % same points.
    %
    % Author : Jean Feydy
    
    param = p;
    [sx, sy] = size(im);
    % Now, we compare several line computing methods.
    param.background.image = 'none';    % remove background
    param.mask.start.method = 'left-top';% deterministic tracing
    param.n_lines = n_lines;                % as many lines as needed
    param.mask.start.initialize.type = 'set';
    %       param.mask.start.initialize.set_value : matrix
    param.mask.start.initialize.set_value = zeros(512, 512);
    X = ceil([sx * rand(1,n_lines); sy * rand(1,n_lines) ]);
    Xlin = sub2ind([sx sy], X(2,:), X(1,:));
    param.mask.start.initialize.set_value(Xlin) = 1;
    hold on; scatter(X(1,:), X(2,:), 81, 'b', 'Marker', 'o'); hold off;

    param.display.scatter = 0;
    param.display.area_param = 'const';
    param.display.area_fun = @(x) 1;
    param.display.color_param = 'const';
    param.display.color_fun = @(x) [1;1;1];

    param.computeline.method = 'cauchy';
    param.display.linespec = '-b';
    param.computeline.time_step = 0.01;
    param.computeline.max_line_size = param.computeline.max_line_size * 10; % To get the same length as with the yellow curve.
    DispGradientLines(im, param);
    param.computeline.max_line_size = param.computeline.max_line_size / 10;
    hold on;
    h = plot([0], [0],  param.display.linespec);
    leg = {'Method : Cauchy, dt = 0.01'};
    hold off;
    
    param.computeline.method = 'cauchy';
    param.display.linespec = '-y';
    param.computeline.time_step = 0.1;
    DispGradientLines(im, param);
    hold on;
    h = [h, plot([0], [0],  param.display.linespec)];
    leg = [leg, 'Method : Cauchy, dt = 0.1'];
    hold off;

    param.computeline.method = 'explicit-piecewise';
    param.display.linespec = '-r';
    DispGradientLines(im, param);
    hold on;
    h = [h, plot([0], [0],  param.display.linespec)];
    leg = [leg, 'Method : Explicit Piecewise'];
    hold off;
    
    param.computeline.method = 'runge-kutta-23';
    param.display.linespec = '-m';
    DispGradientLines(im, param);
    hold on;
    h = [h, plot([0], [0],  param.display.linespec)];
    leg = [leg, 'Method : Runge-Kutta 2-3'];
    hold off;
    
    param.computeline.method = 'runge-kutta-45';
    param.display.linespec = '-g';
    param.computeline.max_line_size = param.computeline.max_line_size * 5; % RK45 typically uses more points per pixel
    DispGradientLines(im, param);
    param.computeline.max_line_size = param.computeline.max_line_size / 5;
    hold on;
    h = [h, plot([0], [0],  param.display.linespec)];
    leg = [leg, 'Method : Runge-Kutta 4-5'];
    hold off;
    
    hold on;
    legend(h, leg);
    hold off;
end