function m = InitializeMask(im, grad, param)
    % Initialize the two masks.
    % Mask is a 2-matrix structure indicating whether or not
    % it might be interesting to start-continue a line at the
    % given location.
    % 1 : ok
    % 0 : nope
    % param may imply that critical points are at first more interesting,
    % etc.
    %   param :
    %       param.mask.start.initialize.type = {'ones', 'topological',
    %           'set'}
    %       param.mask.start.initialize.set_value : matrix
    % Author : Jean Feydy
    
    
    m.continue   = ones(size(im));
    if strcmp(param.mask.start.initialize.type, 'ones')
        m.start      = ones(size(im));
    elseif strcmp(param.mask.start.initialize.type, 'topological')
        %foo = tanh( sqrt( grad(:,:,1).^2 + grad(:,:,2).^2 )/10 ) > tanh(1);
        %m.start = max( GetStrictExtremalPoints(im), ...
        %    0.54 * foo);
        m.start = GetStrictExtremalPoints(im);
    elseif strcmp(param.mask.start.initialize.type, 'set')
        m.start = param.mask.start.initialize.set_value;
    else
        error('param.mask.start.initialize.type does not have a correct value');
    end;
    
    function l = GetStrictExtremalPoints(m)
        % m is a 2-dim double matrix,
        % l is a logical matrix, pointing at the extremal points of m
        %
        % Author : Jean Feydy
        %!!! A modifier !!!
        l = max(imextendedmin(m, 5), ...
                imextendedmax(m, 5));
%         f_t = [[0,1,0];...
%                 [0,-1,0];...
%                 [0,0,0]];
%         f_r = [[0,0,0];...
%                 [0,-1,1];...
%                 [0,0,0]];  
%         f_b = [[0,0,0];...
%                 [0,-1,0];...
%                 [0,1,0]];
%         f_l = [[0,0,0];...
%                 [1,-1,0];...
%                 [0,0,0]];
%         l = isequal(sign(imfilter(m, f_t)),...
%                     sign(imfilter(m, f_r)),...
%                     sign(imfilter(m, f_b)),...
%                     sign(imfilter(m, f_l)));
    end
end