%% Macroscopic Test of the Gradlines software
% Author : Jean Feydy

%% Initialization
close all;
clear all;
FOLDER = 'Output/Rapport/';
RESOLUTION = '-r200';
param = GetDefaultParameters();
im = imread('Images/standard_test_images/lena_gray_512.tif');
im2 = imread('Images/jotunheimen_256.png');
im2 = rgb2gray(im2);
fig = figure('Visible', 'off');

lissage = 0;
points_departs = 0;
longueur_lignes = 0;
densite_lignes = 0;
jotunheimen = 0;
film_rotation = 0;
comparaison_methodes = 1;

% Explicit piecewise is the faster method : we use it.
param.computeline.method = 'explicit-piecewise';
param.grad.interpolation_method = 'piecewise';
% This is actually default, but we just want to remind it.

%% LISSAGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if lissage
    param = GetDefaultParameters();
    param.background.image = 'processed image';
    % We start from a deterministic grid, in order to compare the methods
    param.n_lines = 100; % Actually, we will draw ~64 lines, as lena is 512x512
    param.mask.start.method = 'left-top';
    param.mask.start.filter = zeros(129, 129);
    param.display.linewidth = 2;
    
    param.mask.continue.filter = [0]; % Just stop if lines touch each other
    
    %% Bruitee
    param.grad.processing_method = 'none';
    param.display.scatter = 0;
    
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'bruitee.png'])
    
    %% lissage-gaussien
    param.im.correlation = 1;
    param.grad.processing_method = 'none';
    param.display.scatter = 0;
    
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'lissage-gaussien.png'])
    param.im.correlation = 0;
    
    %% lissage-median
    param.im.median = 1;
    param.grad.processing_method = 'none';
    param.display.scatter = 0;
    
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'lissage-median.png'])
    param.im.median = 0;
    
    %% lissage-wiener
    param.im.wiener = 1;
    param.grad.processing_method = 'none';
    param.display.scatter = 0;
    
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'lissage-wiener.png'])
    param.im.wiener = 0;
    
    %% lissage-none-gaussien
    param.grad.processing_method = 'correlation';
    param.display.scatter = 0;
    
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'lissage-none-gaussien.png'])
    
    %% lissage-none-median
    param.grad.processing_method = 'median';
    param.display.scatter = 0;
    
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'lissage-none-median.png'])
    
    %% lissage-median-gaussien
    param.im.median = 1;
    param.grad.processing_method = 'correlation';
    param.display.scatter = 0;
    
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'lissage-median-gaussien.png'])
    param.im.median = 0;
    
    %% lissage-median-median
    param.im.median = 1;
    param.grad.processing_method = 'median';
    param.display.scatter = 0;
    
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'lissage-median-median.png'])
    param.im.median = 0;
end;

%% POINTS DE DEPART
if points_departs
    param = GetDefaultParameters();
    param.background.image = 'processed image';
    % We start from a deterministic grid, in order to compare the methods
    param.n_lines = 100; % Actually, we will draw ~64 lines, as lena is 512x512
    param.mask.start.filter = zeros(129, 129);
    param.display.linewidth = 2;
    param.mask.continue.filter = [0]; % Just stop if lines touch each other
    
    % Lissage median-gaussien, qui produit de bonnes lignes.
    param.im.median = 1;
    param.grad.processing_method = 'correlation';
    param.display.scatter = 0;
    %% Grille
    param.mask.start.method = 'left-top';
    param.mask.start.initialize.type = 'ones';
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'grille.png'])
    %% Random
    param.mask.start.method = 'random';
    param.mask.start.initialize.type = 'ones';
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'aleatoire.png'])
    %% Topologie
    param.mask.start.method = 'random';
    param.mask.start.initialize.type = 'topological';
    param.mask.start.filter = zeros(5,5);
    param.mask.continue.filter = zeros(1,1);
    param.n_lines = 1000; % Actually, we will draw ~64 lines, as lena is 512x512
    param.display.linewidth = 1;
    
    param.background.image = 'processed image';
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'topologie.png'])
    
    param.n_lines = 0;
    param.background.image = 'start mask';
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'topologie-background.png'])
end;

%% LONGUEUR LIGNES
if longueur_lignes
    param = GetDefaultParameters();
    param.background.image = 'processed image';
    % We start from a deterministic grid, in order to compare the methods
    param.n_lines = 400; % Actually, we will draw ~256 lines, as lena is 512x512
    param.mask.start.filter = zeros(65, 65);
    param.display.linewidth = 2;
    
    % Lissage median-gaussien, qui produit de bonnes lignes.
    param.im.median = 1;
    param.grad.processing_method = 'correlation';
    param.display.scatter = 0;
    
    param.mask.start.method = 'left-top';
    param.mask.start.initialize.type = 'ones';

    %% LIGNES COURTES
    clf;
    param.mask.continue.filter = [0]; % Just stop if lines touch each other
    param.computeline.stopping.time.type = 'arc-length';
    param.computeline.stopping.time.val = 25; % en fait, 2*25
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'lignes-courtes.png'])

    %% LIGNES LONGUES
    clf;
    param.mask.continue.filter = [0]; % Just stop if lines touch each other
    param.computeline.stopping.time.type = 'arc-length';
    param.computeline.stopping.time.val = 250; % en fait, 2*250
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'lignes-longues.png'])

    %% SAC DE NOEUDS
    clf;
    param.mask.start.filter = zeros(7, 7);
    param.n_lines = 1600;
    param.display.linewidth = 1;
    param.computeline.max_line_size = 1000;
    % We restrict ourselves to a subpart of our image, to save time...
    param.mask.start.initialize.type = 'set';
    param.mask.start.initialize.set_value = zeros(size(im));
    param.mask.start.initialize.set_value(101:300, 101:300) = ones(200, 200);
    
    param.mask.continue.filter = [1]; % Don't stop...
    param.computeline.stopping.time.type = 'arc-length';
    param.computeline.stopping.time.val = 2500; % en fait, 2*2500
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'sac_de_noeuds.png'])
    
    %% TRANSPARENCE
    clf;
    param.display.line_alpha = 0.1;
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'transparence.png'])
    param.display.line_alpha = 1;

    %% MASQUAGE - 3px
    clf;
    param.mask.start.initialize.type = 'ones';
    param.mask.start.filter = zeros(33, 33);

    param.mask.continue.filter = fspecial('disk', 3) == 0; % Radius 5
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'masquage-narrow.png'])
    
    %% MASQUAGE - 9px
    clf;
    param.mask.continue.filter = fspecial('disk', 9) == 0; % Radius 10
    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'masquage-wide.png'])
end;

%% DENSITE DES LIGNES
if densite_lignes
    param = GetDefaultParameters();
    param.background.image = 'processed image';
    param.display.linewidth = 2;
    
    % Lissage median-gaussien, qui produit de bonnes lignes.
    param.im.median = 1;
    param.grad.processing_method = 'correlation';
    
    param.mask.start.method = 'left-top';
    param.mask.start.initialize.type = 'ones';
    
%     %% densite faible
%     param.n_lines = 400; % actually, we will draw ~256 lines, as lena is 512x512
%     param.mask.start.filter = zeros(129, 129);
%     param.mask.continue.filter = fspecial('disk', 10) == 0; % radius 10
%     param.display.scatter = 0;
%     
%     dispgradientlines(im, param);
%     print(fig, resolution, '-dpng', [folder, 'densite-faible.png'])
%     
%     
%     %% densite forte
%     param.n_lines = 1500; % actually, we will draw ~1024 lines, as lena is 512x512
%     param.mask.start.filter = zeros(33, 33);
%     param.mask.continue.filter = fspecial('disk', 3) == 0; % radius 10
%     param.display.scatter = 0;
%     
%     dispgradientlines(im, param);
%     print(fig, resolution, '-dpng', [folder, 'densite-forte.png'])
% 
    %% DENSITE SATISFAISANTE
    param.n_lines = 800; % Actually, we will draw ~256 lines, as lena is 512x512
    param.mask.start.method = 'random';
    param.mask.start.filter = fspecial('disk', 30) == 0;
    param.mask.continue.filter = fspecial('disk', 3) == 0; % Radius 10
    param.display.scatter = 1; % Tant qu'a faire, on utilise l'image pour la couleur
    param.display.line = 0;
    param.computeline.max_line_size = 5000;


    DispGradientLines(im, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'densite-satisfaisante.png'])
end;

%% JOTUNHEIMEN
if jotunheimen
    param = GetDefaultParameters();
    param.background.image = 'processed image';

    param.im.median = 0;
    param.grad.processing_method = 'correlation';
    param.display.scatter = 0;
    
    param.mask.start.method = 'left-top';
    param.mask.start.filter = zeros(7, 7);
    param.n_lines = 2500;
    param.display.linewidth = 2;
    param.computeline.max_line_size = 1000;
    % We restrict ourselves to a subpart of our image, to save time...
    param.mask.start.initialize.type = 'ones';
    param.mask.continue.filter = [1]; % Don't stop...
    param.computeline.stopping.time.type = 'arc-length';
    param.computeline.stopping.time.val = 2500; % en fait, 2*2500
    param.computeline.direction = 'backward';

    clf;
    param.display.line_alpha = 0.1;
    DispGradientLines(im2, param);
    print(fig, RESOLUTION, '-dpng', [FOLDER, 'jotunheimen.png'])
    param.display.line_alpha = 1;    
end;

%% FILM
if film_rotation
    param = GetDefaultParameters();
    param.background.image = 'processed image';
    % We start from a deterministic grid, in order to compare the methods
    param.n_lines = 400; % Actually, we will draw ~256 lines, as lena is 512x512
    %param.mask.start.filter = zeros(65, 65);    
    param.mask.start.filter = zeros(150, 150);
    param.display.linewidth = 2;
    
    % Lissage median-gaussien, qui produit de bonnes lignes.
    param.im.median = 1;
    param.grad.processing_method = 'correlation';
    param.display.scatter = 0;
    
    param.mask.start.method = 'left-top';
    param.mask.start.initialize.type = 'ones';
    
    param.mask.start.filter = fspecial('disk', 25) == 0;
    param.mask.continue.filter = fspecial('disk', 3) == 0; % Radius 3
    
    for i = 0:2.8125:180
        param.grad.pre_rotation = i;
        clf;
        DispGradientLines(im, param);
        print(fig, RESOLUTION, '-dpng', [FOLDER, sprintf('film-rotation-%d.png', round(i))])
    end;
end;


%% COMPARAISON METHODES
if comparaison_methodes
    set(fig, 'Visible', 'On');
    param = GetDefaultParameters();
    param.background.image = 'processed image';
    param.grad.interpolation_method = 'piecewise';
    %param.im.median = 1;
    param.grad.processing_method = 'correlation';
    param.display.scatter = 0;
    
    param.computeline.max_line_size = 250;
    
    param.n_lines = 0;
    param.display.quiver = 1;
    DispGradientLines(im, param);
    
    param.display.quiver = 0;
    CompareIntegrationMethods(im, param, 20);
    set(gcf,'renderer','opengl');
end;
%%

% param = GetDefaultParameters();
% param.im.median = 0;
% param.grad.processing_method = 'median';
% param.grad.median_size = [5 5];
% im = imread('Images/jotunheimen_512.png');
% im = rgb2gray(im);
% TestMacroscopicParameters(im, param, 'Output/Macroscopic/Jotunheimen_gradmedian/');

%
% fig = figure; %('Visible', 'off');
%
% % We display the processed image
% param.background.image = 'processed image';
% param.grad.interpolation_method = 'bilinear';
% param.im.median = 1;
% param.grad.processing_method = 'median';
% param.display.line_alpha = 0.1;
% param.display.scatter = 0;
% param.computeline.method = 'explicit-piecewise';
%
% param.mask.start.method = 'random';
% param.mask.start.filter = fspecial('disk', 3) == 0;
% param.mask.continue.filter = [1];
%
% param.n_lines = 4000;
% DispGradientLines(im, param);
% set(gcf,'renderer','opengl');

%print(fig, '-r800', '-dpng', sprintf('Output/lena-test-affichage.png'))


% % We display the processed image
% param.background.image = 'processed image';
% param.grad.interpolation_method = 'bilinear';
% param.im.median = 1;
% param.grad.median_size = [5 5];
%
%
% param.n_lines = 0;
% param.display.quiver = 1;
% DispGradientLines(im, param);
%
% param.display.quiver = 0;
% CompareIntegrationMethods(im, param, 20);
% set(gcf,'renderer','opengl');

